import 'dart:io' as io;

import 'package:import_lint/issue.dart';
import 'package:import_lint/rule.dart';
import 'package:yaml/yaml.dart' as yaml;

Future<void> main(List<String> arguments) async {
  final packageName = await _packageName();

  try {
    final paths = await _dartFilePaths();
    final rules = await _loadYamlFile();
    final issues = await _checkIssues(rules: rules, paths: paths);

    final output = await _output(packageName: packageName, issues: issues);

    io.stdout.writeln(output);
    io.exit(0);
  } catch (e) {
    io.stdout.writeln('😿' + e.toString().replaceAll('Exception: ', ''));
    io.exit(1);
  }
}

Future<String> _packageName() async {
  final pubspecFile = io.File("pubspec.yaml");
  final value = await pubspecFile.readAsString();
  final loadYaml = yaml.loadYaml(value);
  return loadYaml['name'];
}

Future<Rules> _loadYamlFile() async {
  final file = io.File('import_analysis_options.yaml');
  if (!await file.exists()) {
    throw Exception(
      'Not Found: import_analysis_options.yaml file at the root of your project.',
    );
  }
  final value = await file.readAsString();
  final loadYaml = yaml.loadYaml(value);
  return Rules.fromParsedYaml(loadYaml);
}

Future<List<String>> _dartFilePaths() async {
  final dic = io.Directory.current;
  final dartFileRegExp = RegExp('.*dart');
  final result = <String>[];
  await for (final entry in dic.list(recursive: true, followLinks: false)) {
    if (dartFileRegExp.hasMatch(entry.path)) {
      result.add(entry.path);
    }
  }
  return result;
}

Future<List<Issue>> _checkIssues({
  required Rules rules,
  required List<String> paths,
}) async {
  final issues = <Issue>[];
  for (final rule in rules.value) {
    for (final path in paths) {
      if (rule.searchFilePathRegExp.hasMatch(path)) {
        final lines = await io.File(path).readAsLines();
        if (lines.isNotEmpty) {
          final fileIssues = Issues.ofRuleAndLines(
            rule: rule,
            filePath: path,
            lines: lines,
          );
          if (fileIssues.value.isNotEmpty) {
            issues.addAll(fileIssues.value);
          }
        }
      }
    }
  }
  return issues;
}

Future<String> _output({
  required String packageName,
  required List<Issue> issues,
}) async {
  if (issues.isEmpty) {
    return 'No issues found! 🎉';
  }

  final currentDic = io.Directory.current;

  final buffer = StringBuffer();

  for (final issue in issues) {
    for (final line in issue.lines) {
      final modFilePath =
          issue.filePath.replaceAll(currentDic.path, '').replaceAll('lib/', '');
      final modLine = line.value.replaceAll(';', '');
      buffer.write(
        '   ${issue.rule.name} • package:$packageName$modFilePath • $modLine:${line.number} \n',
      );
    }
  }

  return buffer.toString();
}
